@@ -181,18 +181,20 @@ impl From<log::Level> for Priority {
 #[derive(Clone, Copy, Debug)]
 #[repr(u8)]
 pub enum Buffer {
-    /// Main Buffer with id 0
+    /// The main log buffer. This is the only log buffer available to apps.
     Main,
-    /// Radio Buffer with id 1
+    /// The radio log buffer
     Radio,
-    /// Event Buffer with id 2
+    /// The event log buffer.
     Events,
-    /// System Buffer with id 3
+    /// The system log buffer.
     System,
-    /// Crash Buffer with id 4
+    /// The crash log buffer.
     Crash,
-    /// Kernel Buffer with id 5
-    Kernel,
+    /// The statistics log buffer.
+    Stats,
+    /// The security log buffer.
+    Security,
     /// User defined Buffer
     Custom(u8),
 }
@@ -205,7 +207,8 @@ impl From<Buffer> for u8 {
             Buffer::Events => 2,
             Buffer::System => 3,
             Buffer::Crash => 4,
-            Buffer::Kernel => 5,
+            Buffer::Stats => 5,
+            Buffer::Security => 6,
             Buffer::Custom(id) => id,
         }
     }
@@ -539,9 +542,6 @@ pub fn builder<'a>() -> Builder<'a> {
 /// Event tag
 pub type EventTag = u32;
 
-/// Event name
-pub type EventName = String;
-
 /// Event data
 #[derive(Debug, Clone, PartialEq)]
 pub struct Event {
@@ -556,6 +556,8 @@ pub struct Event {
 /// Event's value
 #[derive(Debug, PartialEq, Clone)]
 pub enum EventValue {
+    /// Void value
+    Void,
     /// Int value
     Int(i32),
     /// Long value
@@ -572,6 +574,7 @@ impl EventValue {
     /// Serialied size
     pub fn serialized_size(&self) -> usize {
         match self {
+            &EventValue::Void => 0,
             EventValue::Int(_) | EventValue::Float(_) => 1 + 4,
             EventValue::Long(_) => 1 + 8,
             EventValue::String(s) => 1 + 4 + s.as_bytes().len(),
@@ -589,6 +592,7 @@ impl EventValue {
 
         let mut buffer = BytesMut::with_capacity(self.serialized_size());
         match self {
+            EventValue::Void => (),
             EventValue::Int(num) => {
                 buffer.put_u8(EVENT_TYPE_INT);
                 buffer.put_i32_le(*num);
@@ -616,6 +620,12 @@ impl EventValue {
     }
 }
 
+impl From<()> for EventValue {
+    fn from(_: ()) -> Self {
+        EventValue::Void
+    }
+}
+
 impl From<i32> for EventValue {
     fn from(v: i32) -> Self {
         EventValue::Int(v)
@@ -658,7 +668,7 @@ where
     }
 }
 
-/// Write an event with the timestamp now
+/// Write an event with the timestamp now to `Buffer::Events`
 /// ```
 /// use android_logd_logger::{write_event, write_event_now, Error, Event, EventValue};
 /// android_logd_logger::builder().init();
@@ -676,9 +686,30 @@ pub fn write_event_now<T: Into<EventValue>>(tag: EventTag, value: T) -> Result<(
     })
 }
 
-/// Write an event
+/// Write an event with the timestamp now to buffer
 /// ```
-/// use android_logd_logger::{write_event, write_event_now, Error, Event, EventValue};
+/// use android_logd_logger::{write_event_buffer_now, Buffer, Error, Event, EventValue};
+/// android_logd_logger::builder().init();
+///
+/// write_event_buffer_now(Buffer::Stats, 1, "test").unwrap();
+///
+/// let value: Vec<EventValue> = vec![1.into(), "one".into(), 123.3.into()].into();
+/// write_event_buffer_now(Buffer::Stats, 2, value).unwrap();
+/// ```
+pub fn write_event_buffer_now<T: Into<EventValue>>(log_buffer: Buffer, tag: EventTag, value: T) -> Result<(), Error> {
+    write_event_buffer(
+        log_buffer,
+        &Event {
+            timestamp: SystemTime::now(),
+            tag,
+            value: value.into(),
+        },
+    )
+}
+
+/// Write an event to `Buffer::Events`
+/// ```
+/// use android_logd_logger::{write_event, Error, Event, EventValue};
 /// android_logd_logger::builder().init();
 ///
 /// write_event(&Event {
@@ -688,6 +719,21 @@ pub fn write_event_now<T: Into<EventValue>>(tag: EventTag, value: T) -> Result<(
 /// }).unwrap();
 /// ```
 pub fn write_event(event: &Event) -> Result<(), Error> {
+    write_event_buffer(Buffer::Events, event)
+}
+
+/// Write an event to an explicit buffer
+/// ```
+/// use android_logd_logger::{write_event_buffer, Buffer, Error, Event, EventValue};
+/// android_logd_logger::builder().init();
+///
+/// write_event_buffer(Buffer::Stats, &Event {
+///     timestamp: std::time::SystemTime::now(),
+///     tag: 1,
+///     value: "blah".into(),
+/// }).unwrap();
+/// ```
+pub fn write_event_buffer(log_buffer: Buffer, event: &Event) -> Result<(), Error> {
     if event.value.serialized_size() > (LOGGER_ENTRY_MAX_LEN - 1 - 2 - 4 - 4 - 4) {
         return Err(Error::EventSize);
     }
@@ -697,7 +743,7 @@ pub fn write_event(event: &Event) -> Result<(), Error> {
         let mut buffer = bytes::BytesMut::with_capacity(LOGGER_ENTRY_MAX_LEN);
         let timestamp = event.timestamp.elapsed().unwrap();
 
-        buffer.put_u8(Buffer::Events.into());
+        buffer.put_u8(log_buffer.into());
         buffer.put_u16_le(thread::id() as u16);
         buffer.put_u32_le(timestamp.as_secs() as u32);
         buffer.put_u32_le(timestamp.subsec_nanos());
@@ -712,7 +758,7 @@ pub fn write_event(event: &Event) -> Result<(), Error> {
     }
 
     #[cfg(not(target_os = "android"))]
-    println!("event: {:?}", event);
+    println!("buffer: {:?}, event: {:?}", log_buffer, event);
 
     Ok(())
 }
